﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DFU_Example
{
    public partial class MainForm : Form
    {
        IFirmwareUpdate firmwareUpdate = new FirmwareUpdate();

        public MainForm()
        {
            InitializeComponent();
        }

        private void Program()
        {
            UInt16 VID;
            UInt16 PID;
            UInt16 Version;

            try
            {
                timer.Enabled = false;
                if (chbAutoErase.Checked)
                {
                    firmwareUpdate.MassErase();
                }
                firmwareUpdate.OnFirmwareUpdateProgress += FirmwareUpdateProgressHandler;
                firmwareUpdate.ParseDFU_File(Properties.Settings.Default.DfuFile, out VID, out PID, out Version);

                // Check that the file's VID, PID and Version are OK, then update
                firmwareUpdate.UpdateFirmware(Properties.Settings.Default.DfuFile, false);
                if (Properties.Settings.Default.BeepOnProgramming)
                    Console.Beep();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                timer.Enabled = chbAutoProgram.Checked;
            }
        }

        private void btnProgram_Click(object sender, EventArgs e)
        {
            Program();
        }

        private void btnMassErase_Click(object sender, EventArgs e)
        {
            try
            {
                timer.Enabled = false;
                firmwareUpdate.MassErase();
            }
            finally
            {
                timer.Enabled = chbAutoProgram.Checked;
            }
        }

        private void FirmwareUpdateProgressHandler(object sender, FirmwareUpdateProgressEventArgs e)
        {
            if (Progress.InvokeRequired)
            {
                Invoke(new EventHandler<FirmwareUpdateProgressEventArgs>(FirmwareUpdateProgressHandler), new object[] { sender, e });
            }
            else
            {
                lblMessage.Text = e.Message;
                Progress.Value = (int)e.Percentage;
                if (e.Failed)
                {
                    MessageBox.Show(e.Message, "Firmware update failed");
                }
            }
        }

        private void chbAutoProgram_CheckedChanged(object sender, EventArgs e)
        {
            timer.Enabled = chbAutoProgram.Checked;
            lblState.Visible = chbAutoProgram.Checked;
        }

        private int connectedDevicesCnt = 0;

        private void timer_Tick(object sender, EventArgs e)
        {
            try
            {
                timer.Enabled = false;
                lblState.Visible = true;
                int cnt = firmwareUpdate.CheckDevicesConnected();
                if (cnt == 1)
                {
                    lblState.Text = "Connected";
                    if (connectedDevicesCnt == 0)
                    {
                        if (chbAutoProgram.Checked)
                            Program();
                    }
                }
                else if (cnt == 0)
                {
                    lblState.Text = "Not connected";
                }
                else
                {
                    lblState.Text = "More than one DFU device connected!";
                }
                connectedDevicesCnt = cnt;
            }
            finally
            {
                timer.Enabled = true;
            }
        }

        private void btnSelectDfuFile_Click(object sender, EventArgs e)
        {
            if (DialogResult.OK == dlgOpenFile.ShowDialog())
            {
                Properties.Settings.Default.DfuFile = dlgOpenFile.FileName;
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.Save();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }
    }
}
